; Hide a DOS extended partition by replacing the partition type with E7h.
; An LBA extended partition is replaced with type E8h.
; Partition types are as follows:
;   DOS Extended partition = 05h
;   DOS LBA Extended partition = 0Fh

.model tiny

code		segment
		assume cs:code, ds:code, es:code
		org 0100h

prog_start:
		mov bp, offset buf

		mov ax, 0201h			; read 1 sector
		mov cx, 1			; sector 1
		xor dh, dh			; head 0
		mov dl, drive
		mov bx, bp			; data buffer
		int 13h				; read MBR
		mov si, offset msg_err_read
		jc error

		; look for partition in partition table
		mov cx, 4
		mov si, 01c2h
part_loop:
		cmp byte ptr [bp+si], 5		; DOS extended partition
		jz found_ext_part
		cmp byte ptr [bp+si], 0fh	; DOS extended (LBA)
		jz found_lba_ext_part
		add si, 010h
		loop part_loop

		; did not find partition - print message and exit
		mov si, offset msg_not_found
		call PrintStr
		jmp prog_exit

found_ext_part:
		mov byte ptr [bp+si], 0e7h		; replace partition type
		jmp write_mbr

found_lba_ext_part:
		mov byte ptr [bp+si], 0e8h		; replace partition type
		; jmp write_mbr

write_mbr:
		mov ax, 0301h			; write 1 sector
		mov cx, 1			; sector 1
		xor dh, dh			; head 0
		mov dl, drive
		mov bx, bp			; data buffer
		int 13h				; write MBR
		mov si, offset msg_err_write
		jc error

		; print okay message
		mov si, offset msg_okay
		call PrintStr
		
prog_exit:
		mov ax, 4c00h
		int 21h				; exit

error:
		call PrintStr
		jmp prog_exit

;-----------------------------------------------------------------------------

include		PrintStr.inc

;-----------------------------------------------------------------------------

msg_err_read	db "Error reading MBR",10,13,0
msg_err_write	db "Error writing MBR",10,13,0
msg_not_found	db "No extended partitions found",10,13,0
msg_okay	db "Success",10,13,0

buf		equ $

code		ends

drive	=	080h

end		prog_start
